// Copyright (C) 2012 Stefan Prey
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 
// 
// This file is part of kanonprogramme.
// 
// kanonprogramme is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// kanonprogramme is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with kanonprogramme.  If not, see <http://www.gnu.org/licenses/>.
// 
// Diese Datei ist Teil von kanonprogramme.
// 
// kanonprogramme ist Freie Software: Sie koennen es unter den Bedingungen
// der GNU General Public License, wie von der Free Software Foundation,
// Version 3 der Lizenz oder (nach Ihrer Option) jeder spaeteren
// veroeffentlichten Version, weiterverbreiten und/oder modifizieren.
// 
// kanonprogramme wird in der Hoffnung, dass es nuetzlich sein wird, aber
// OHNE JEDE GEWAEHELEISTUNG, bereitgestellt; sogar ohne die implizite
// Gewaehrleistung der MARKTFAEHIGKEIT oder EIGNUNG FUER EINEN BESTIMMTEN ZWECK.
// Siehe die GNU General Public License fuer weitere Details.
// 
// Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
// Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.

// Vielleicht wäre es sinnvoll, nicht nur festzustellen, ob ein weiteres
// Die Funktion tut jetzt wenigstens das, was sie soll.
// Konstruktor mit string funktioniert nicht; ausprobieren, ob es mit char* klappt

#include "lesen.h"

const int lesen::obergrenze = 0x7fff;
const int lesen::untergrenze = -0x8000;

bool lesen::ist_ziffer(char zeichen)
{
    return '0' <= zeichen && zeichen <= '9';
}

bool lesen::ist_zahlzeichen(char zeichen)
{
    return ist_ziffer(zeichen) || zeichen == '-' || zeichen == '+';
}

void lesen::zeichen_lesen()
{
    m_ein >> m_naechstes_zeichen;
    m_ende = m_ein.eof();
    m_fehler = m_ein.fail();
}

lesen::lesen(ifstream& ein):m_ein(ein)
{
    m_ein.unsetf(ios::skipws); // Zwischenräume und Zeilenenden nicht ueberlesen
    m_ende = m_fehler = false;
    zeichen_lesen();
}

lesen::~lesen()
{
}

bool lesen::ende()
{
    return m_ende;
}

bool lesen::fehler()
{
    return m_fehler;
}

bool lesen::existiert()
{
    return !m_ende && !m_fehler;
}

char lesen::naechstes_zeichen()
{
    return m_naechstes_zeichen;
}

void lesen::operator>>(char& zeichen)
{
    zeichen = m_naechstes_zeichen;
    zeichen_lesen();
    return;
}

void lesen::weiss()
{
    zeichen_lesen();
    return;
}

void lesen::kommentar()
{

    zeichen_lesen();
    while(existiert() && m_naechstes_zeichen != '\n')
	zeichen_lesen();
    return;
}

void lesen::zahlein(int& zahl)
{
    int vorzeichen = 1;
    int anzahl_der_ziffern = 0;
    bool ziffern_eingelesen = false;
    zahl = 0;
    if(m_naechstes_zeichen == '-')
    {
	vorzeichen = -1;
	zeichen_lesen();
    }
    else if(m_naechstes_zeichen == '+')
	zeichen_lesen();

    if(vorzeichen == 1) // positive Zahl
    {
	while(existiert() == true && ist_ziffer(m_naechstes_zeichen) && zahl <= (obergrenze - (m_naechstes_zeichen - '0')) / 10)
	{
	    ziffern_eingelesen = true;
	    zahl *= 10;
	    zahl += m_naechstes_zeichen - '0';
	    zeichen_lesen();
	}
	if(ziffern_eingelesen == false || ist_zahlzeichen(m_naechstes_zeichen)) // Keine Ziffer eingelesen oder kein Trenner zur nächsten Zahl
	    m_fehler = true;
	return;
    }
// negative Zahl
    while(existiert() == true && ist_ziffer(m_naechstes_zeichen) && zahl >= (untergrenze + (m_naechstes_zeichen - '0')) / 10)
    {
	ziffern_eingelesen = true;
	zahl *= 10;
	zahl -= m_naechstes_zeichen - '0';
	zeichen_lesen();
    }
    if(ziffern_eingelesen == false || ist_zahlzeichen(m_naechstes_zeichen)) // Keine Ziffer eingelesen oder kein Trenner zur nächsten Zahl
	m_fehler = true;
    return;
}

void lesen::operator>>(int& zahl)
{
    while(existiert())
    {
	switch(m_naechstes_zeichen)
	{
	    case ' ':
	    case '\t':
	    case '\n':
	    case '\r':
		weiss();
	        break;
	    case '#':
		kommentar();
		break;
	    case '0':
	    case '1':
	    case '2':
	    case '3':
	    case '4':
	    case '5':
	    case '6':
	    case '7':
	    case '8':
	    case '9':
	    case '+':
	    case '-':
	    default:
	    	zahlein(zahl);
		return;
	}
    }
}

