// Copyright (C) 2012 Stefan Prey
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
// 
// 
// This file is part of kanonprogramme.
// 
// kanonprogramme is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// kanonprogramme is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with kanonprogramme.  If not, see <http://www.gnu.org/licenses/>.
// 
// Diese Datei ist Teil von kanonprogramme.
// 
// kanonprogramme ist Freie Software: Sie koennen es unter den Bedingungen
// der GNU General Public License, wie von der Free Software Foundation,
// Version 3 der Lizenz oder (nach Ihrer Option) jeder spaeteren
// veroeffentlichten Version, weiterverbreiten und/oder modifizieren.
// 
// kanonprogramme wird in der Hoffnung, dass es nuetzlich sein wird, aber
// OHNE JEDE GEWAEHELEISTUNG, bereitgestellt; sogar ohne die implizite
// Gewaehrleistung der MARKTFAEHIGKEIT oder EIGNUNG FUER EINEN BESTIMMTEN ZWECK.
// Siehe die GNU General Public License fuer weitere Details.
// 
// Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
// Programm erhalten haben. Wenn nicht, siehe <http://www.gnu.org/licenses/>.

// Funktioniert lesen, wenn das letzte Zeichen der Datei eine Ziffer ist?

// Insgesamt sind diese Schritte erforderlich: 
// Die Konstruktion der Lilypond-Datei
// Das Sortieren des Einsatzschemas nach der Tonhoehe kann in die Konstruktion der Lilyponddatei verlagert werden
// Diese Schritte sollten, soweit es möglich ist, aus main() ausgegliedert werden

// Behandlung von Lesefehlern

#include <iostream>
#include <fstream>

using namespace std;

#include "knoten.h"
#include "graph.h"
#include "folge.h"
#include "lesen.h"
#include "stimme.h"
#include "einsatzschema.h"
#include "lilypond_datei.h"
#include "einzelstimme.h"

void schreibe_lilypond_datei(einsatzschema& das_einsatzschema, const folge& die_folge)
{
    lilypond_datei kanon_ausgabe("kanon.ly");

// Einsatzschema sortieren
    das_einsatzschema.sortieren();
    int hoehenkorrektur = die_folge.get_mittelposition() + das_einsatzschema.get_mitte(); // Damit der Satz insgesamt in der Mittellage bleibt
    for(int i = 0; i < das_einsatzschema.get_anzahl_stimmen(); i++)
    {
	einzelstimme die_stimme(kanon_ausgabe, das_einsatzschema.get_stimme_intervallischer_abstand(i) - das_einsatzschema.get_mitte());
	die_stimme.pausen(das_einsatzschema.get_stimme_rhythmischer_abstand(i));
	for(int j = 0; j < die_folge.get_laenge(); j++)
	    die_stimme.ton(die_folge.get_wert(j) + das_einsatzschema.get_stimme_intervallischer_abstand(i) - hoehenkorrektur);
	die_stimme.pausen(das_einsatzschema.get_max_abstand() - das_einsatzschema.get_stimme_rhythmischer_abstand(i));
    }
}

int main()
{
    ifstream eingabe_datei("graphendatei.txt");
    lesen ein(eingabe_datei);

    int anzahl_der_stimmen;
    ein >> anzahl_der_stimmen;
    einsatzschema das_einsatzschema(anzahl_der_stimmen);

    for(int stimmen_nummer = 0; stimmen_nummer < anzahl_der_stimmen; stimmen_nummer++)
    {
	int rh_abstand;
	int ivl_abstand;
	ein >> rh_abstand;
	ein >> ivl_abstand;
	das_einsatzschema.einfuegen(stimmen_nummer, stimme(rh_abstand, ivl_abstand));
    }

    int anzahl_der_knoten;
    ein >> anzahl_der_knoten;

    graph der_graph(anzahl_der_knoten);

    for(int i = 0; i < anzahl_der_knoten; i++)
    {
	int knoten_nummer;
	int knoten_wert;
	int knoten_anzahl_adjazenzen;

	ein >> knoten_nummer;
	ein >> knoten_wert;
	ein >> knoten_anzahl_adjazenzen;

	int* adjazenzliste = new int [knoten_anzahl_adjazenzen];
	if(adjazenzliste == NULL)
	    exit(1);
	for(int j = 0; j < knoten_anzahl_adjazenzen; j++)
	    ein >> adjazenzliste[j];
	der_graph.einfuegen(knoten_nummer, knoten(knoten_wert, knoten_anzahl_adjazenzen, adjazenzliste));
	delete[] adjazenzliste;
    }

    folge die_folge;
    cout << "Zufaellige Folge wird konstruiert" << endl;
    die_folge = der_graph.zufallsfolge(16);
    cout << "Notensatzdatei \"kanon.ly\" wird erstellt" << endl;
    schreibe_lilypond_datei(das_einsatzschema, die_folge);
    cout << "Konvertierung durch Lilypond" << endl;
    system("lilypond kanon.ly");
    return 0;
}
